@@ -16,7 +16,7 @@
 
 import pytest
 
-from google.cloud import bigquery_v2
+from google.cloud import bigquery
 
 
 @pytest.fixture
@@ -27,8 +27,8 @@ def target_class():
 
 
 def test_ctor(target_class):
-    data_type = bigquery_v2.types.StandardSqlDataType(
-        type_kind=bigquery_v2.types.StandardSqlDataType.TypeKind.INT64
+    data_type = bigquery.standard_sql.StandardSqlDataType(
+        type_kind=bigquery.StandardSqlTypeNames.INT64
     )
     actual_arg = target_class(
         name="field_name", kind="FIXED_TYPE", mode="IN", data_type=data_type
@@ -50,8 +50,8 @@ def test_from_api_repr(target_class):
     assert actual_arg.name == "field_name"
     assert actual_arg.kind == "FIXED_TYPE"
     assert actual_arg.mode == "IN"
-    assert actual_arg.data_type == bigquery_v2.types.StandardSqlDataType(
-        type_kind=bigquery_v2.types.StandardSqlDataType.TypeKind.INT64
+    assert actual_arg.data_type == bigquery.standard_sql.StandardSqlDataType(
+        type_kind=bigquery.StandardSqlTypeNames.INT64
     )
 
 
@@ -71,8 +71,8 @@ def test_from_api_repr_w_unknown_fields(target_class):
 
 
 def test_eq(target_class):
-    data_type = bigquery_v2.types.StandardSqlDataType(
-        type_kind=bigquery_v2.types.StandardSqlDataType.TypeKind.INT64
+    data_type = bigquery.standard_sql.StandardSqlDataType(
+        type_kind=bigquery.StandardSqlTypeNames.INT64
     )
     arg = target_class(
         name="field_name", kind="FIXED_TYPE", mode="IN", data_type=data_type
